@@ -1,4 +1,4 @@
-// Copyright 2009 Ben Hutchings.
+// Copyright 2009-2010 Ben Hutchings.
 // See the file "COPYING" for licence details.
 
 // Some versions of ffmpeg define an ABS macro, which glib also does.
@@ -6,6 +6,14 @@
 // provoke a warning.
 #undef ABS
 
+// <avcodec.h> may need UINT64_C while <stdint.h> may or may not
+// define that for C++.  Therefore, include <stdint.h> here and then
+// define UINT64_C if it didn't get defined.
+#include <stdint.h>
+#ifndef UINT64_C
+#define UINT64_C(n) n ## ULL
+#endif
+
 // These guards were removed from <avcodec.h>... what were they thinking?
 #ifdef __cplusplus
 extern "C" {
