@@ -62,9 +62,8 @@ mixer::source_id mixer::add_source(source * src)
     source_id id;
     for (id = 0; id != sources_.size(); ++id)
     {
-	if (!sources_[id].is_live)
+	if (!sources_[id].src)
 	{
-	    sources_[id].is_live = true;
 	    sources_[id].src = src;
 	    return id;
 	}
@@ -77,7 +76,7 @@ mixer::source_id mixer::add_source(source * src)
 void mixer::remove_source(source_id id)
 {
     boost::mutex::scoped_lock lock(source_mutex_);
-    sources_.at(id).is_live = false;
+    sources_.at(id).src = NULL;
 }
 
 void mixer::put_frame(source_id id, const dv_frame_ptr & frame)
@@ -102,8 +101,9 @@ void mixer::put_frame(source_id id, const dv_frame_ptr & frame)
 		&& source.frames.size() == target_queue_len)
 	    {
 		settings_.video_source_id = id;
-		sources_[settings_.video_source_id].src->
-		    set_active(source_active_video);
+		if (sources_[settings_.video_source_id].src)
+		    sources_[settings_.video_source_id].src->
+			set_active(source_active_video);
 		settings_.audio_source_id = id;
 		clock_state_ = run_state_run;
 		should_notify_clock = true; // after we unlock the mutex
@@ -131,11 +131,8 @@ void mixer::put_frame(source_id id, const dv_frame_ptr & frame)
 		if (format_.frame_aspect == dv_frame_aspect_auto)
 		    format_.frame_aspect = format.frame_aspect;
 		else if (format_.frame_aspect != format.frame_aspect)
-		{
-		    std::cerr << "WARN: Source " << 1 + id
-			      << " using wrong frame aspect\n";
-		    frame->format_error = true;
-		}
+		    // Override frame aspect ratio
+		    dv_frame_set_aspect(frame.get(), format_.frame_aspect);
 
 		if (format_.sample_rate == dv_sample_rate_auto)
 		    format_.sample_rate = format.sample_rate;
@@ -211,11 +208,13 @@ void mixer::set_video_source(source_id id)
     boost::mutex::scoped_lock lock(source_mutex_);
     if (id < sources_.size())
     {
-	sources_[settings_.video_source_id].src->
-	    set_active(source_active_none);
+	if (sources_[settings_.video_source_id].src)
+	    sources_[settings_.video_source_id].src->
+		set_active(source_active_none);
 	settings_.video_source_id = id;
-	sources_[settings_.video_source_id].src->
-	    set_active(source_active_video);
+	if (sources_[settings_.video_source_id].src)
+	    sources_[settings_.video_source_id].src->
+		set_active(source_active_video);
     }
     else
 	throw std::range_error("video source id out of range");
@@ -225,11 +224,13 @@ void mixer::set_video_effect(
     std::tr1::shared_ptr<video_effect_settings> effect)
 {
     boost::mutex::scoped_lock lock(source_mutex_);
-    if (settings_.video_effect)
+    if (settings_.video_effect &&
+	sources_[settings_.video_effect->sec_source_id].src)
 	sources_[settings_.video_effect->sec_source_id].src->
 	    set_active(source_active_none);
     settings_.video_effect = effect;
-    if (settings_.video_effect)
+    if (settings_.video_effect &&
+	sources_[settings_.video_effect->sec_source_id].src)
 	sources_[settings_.video_effect->sec_source_id].src->
 	    set_active(source_active_video);
 }
@@ -615,8 +616,18 @@ void mixer::run_mixer()
     dec->release_buffer = raw_frame_release_buffer;
     dec->reget_buffer = raw_frame_reget_buffer;
 
-    auto_codec encoder(auto_codec_open_encoder(CODEC_ID_DVVIDEO));
+    auto_codec encoder(avcodec_alloc_context());
     AVCodecContext * enc = encoder.get();
+    if (!enc)
+	throw std::bad_alloc();
+    // Set some input parameters early to satisfy
+    // dvvideo_init_encoder() that the input will match some DV
+    // profile.  It doesn't matter if we change these later.
+    enc->width = 720;
+    enc->height = 576;
+    enc->pix_fmt = PIX_FMT_YUV420P;
+    auto_codec_open_encoder(encoder, CODEC_ID_DVVIDEO);
+
     {
 	// Try to use one thread per CPU, up to a limit of 8
 	int enc_thread_count =
@@ -718,6 +729,16 @@ void mixer::run_mixer()
 						&mixed_raw->header);
 	    assert(size_t(out_size) == system->size);
 	    mixed_dv->serial_num = serial_num;
+
+	    // libavcodec doesn't properly distinguish IEC and SMPTE
+	    // variants of NTSC.  Fix the APTs here.
+	    if (system == &dv_system_525_60)
+	    {
+		uint8_t * block = mixed_dv->buffer;
+		unsigned apt = 0;
+		for (unsigned i = 4; i != 8; ++i)
+		    block[i] = (block[i] & 0xf8) | apt;
+	    }
 	}
 	else
 	{
